# This is the MNIST example from the h2O Deep Learning Booklet

# https://h2o-release.s3.amazonaws.com/h2o/rel-xia/1/docs-website/h2o-docs/booklets/DeepLearningBooklet.pdf

library(h2o)

# Sets number of threads to number of available cores
h2o.init(nthreads = -1)

train_file <- "https://h2o-public-test-data.s3.amazonaws.com/bigdata/laptop/mnist/train.csv.gz"
test_file <- "https://h2o-public-test-data.s3.amazonaws.com/bigdata/laptop/mnist/test.csv.gz"

train <- h2o.importFile(train_file)
test <- h2o.importFile(test_file)

# Specify the response and predictor columns
y <- "C785"
x <- setdiff(names(train), y)

# Encode the response column as categorical formultinomial classification
train[,y] <- as.factor(train[,y])
test[,y] <- as.factor(test[,y])

# Train Deep Learning model and validate on test set
model <- h2o.deeplearning(
 x = x,
 y = y,
 training_frame = train,
 validation_frame = test,
 distribution = "multinomial",
 activation = "RectifierWithDropout",
 hidden = c(32,32,32),
 input_dropout_ratio = 0.2,
 sparse = TRUE,
 l1 = 1e-5,
 epochs = 10)

# Perform 5-fold cross-validation on training_frame
model_cv <- h2o.deeplearning(
 x = x,
 y = y,
 training_frame = train,
 distribution = "multinomial",
 activation = "RectifierWithDropout",
 hidden = c(32,32,32),
 input_dropout_ratio = 0.2,
 sparse = TRUE,
 l1 = 1e-5,
 epochs = 10,
 nfolds = 5)

# View specified parameters of the deep learning model
model@parameters

# Examine the performance of the trained model
model # display all performance metrics

h2o.performance(model) # training metrics
h2o.performance(model, valid = TRUE) # validation metrics

# Get MSE only
h2o.mse(model, valid = TRUE)

# Cross-validated MSE
h2o.mse(model_cv, xval = TRUE)

# Classify the test set (predict class labels)
# This also returns the probability for each class
pred <- h2o.predict(model, newdata = test)

# Take a look at the predictions
test[,y]
head(pred[,1])

# Retrieve the variable importance
h2o.varimp(model)

# Grid Search

hidden_opt <- list(c(32,32), c(32,16,8), c(100))
l1_opt <- c(1e-4,1e-3)
hyper_params <- list(hidden = hidden_opt, l1 = l1_opt)

model_grid <- h2o.grid("deeplearning",
                          grid_id = "mygrid",
                          hyper_params = hyper_params,
                          x = x,
                          y = y,
                          distribution = "multinomial",
                          training_frame = train,
                          validation_frame = test,
                          score_interval = 2,
                          epochs = 1000,
                          stopping_rounds = 3,
                          stopping_tolerance = 0.05,
                          stopping_metric = "misclassification")

# print out all prediction errors and run times of the models
model_grid

# print out the Test MSE for all of the models
for (model_id in model_grid@model_ids) {
 mse <- h2o.mse(h2o.getModel(model_id), valid = TRUE)
 print(sprintf("Test set MSE: %f", mse))
}

# Random Grid Search
 
hidden_opt = lapply(1:100, function(x) 10+sample(50, sample(4), replace=TRUE))
l1_opt = seq(1e-6,1e-3,1e-6)
hyper_params <- list(hidden = hidden_opt, l1 = l1_opt)
search_criteria = list(strategy = "RandomDiscrete",
                         max_models = 10, max_runtime_secs = 100,
                         seed=123456)
 
model_grid <- h2o.grid("deeplearning",
                           grid_id = "mygrid",
                           hyper_params = hyper_params,
                           search_criteria = search_criteria,
                           x = x,
                           y = y,
                           distribution = "multinomial",
                           training_frame = train,
                           validation_frame = test,
                           score_interval = 2,
                           epochs = 1000,
                           stopping_rounds = 3,
                           stopping_tolerance = 0.05,
                           stopping_metric = "misclassification")

# print out all prediction errors and run times of the models
model_grid

# print out the Test MSE for all of the models
for (model_id in model_grid@model_ids) {
  mse <- h2o.mse(h2o.getModel(model_id), valid = TRUE)
  print(sprintf("Test set MSE: %f", mse))
}
 
# World Record run used epochs=8000  !!!!!! This will take many many hours !!!!!! 

model <- h2o.deeplearning(x=x, y=y,
                            training_frame=train, validation_frame=test,
                            activation="RectifierWithDropout",
                            hidden=c(1024,1024,2048), epochs=10,
                            input_dropout_ratio=0.2, l1=1e-5, max_w2=10,
                            train_samples_per_iteration=-1,
                            classification_stop=-1, stopping_rounds=0)

model 

h2o.shutdown()


